



/* 初始化缓冲区结构 */
void Init_Buffer(struct prodcons *buffer)
{
    pthread_mutex_init(&buffer->lock,NULL);
    pthread_cond_init(&buffer->notempty,NULL);
    pthread_cond_init(&buffer->notfull,NULL);

    buffer->readpos=0;
    buffer->writepos=0;
    buffer->size=0;
}

/* 将产品放入缓冲区，这里存入的是一个整数 */
void put(struct prodcons*buffer,int data)
{
    pthread_mutex_lock(&buffer->lock);

    /* 等待缓冲区未满 */
    while(buffer->size==BUFFER_SIZE)
    {
        pthread_cond_wait(&buffer->notfull,&buffer->lock);
    }

    buffer->buffer[buffer->writepos]=data; /* 写入数据 */
    buffer->writepos++;                    /* 移动写指针 */
    if (buffer->writepos>=BUFFER_SIZE)     /* 判断是否溢出 */
    {
        buffer->writepos=0;
    }
    buffer->size++;

    /* 设置缓冲区非空的条件变量 */
    pthread_cond_signal(&buffer->notempty);
    pthread_mutex_unlock(&buffer->lock);
    
}

/* 从缓冲区中读取数据 */
int get(struct prodcons *b)
{
    int data;
    pthread_mutex_lock(&b->lock);

    /* 等待缓冲区非空 */
    while(b->size==0)
    {
        printf ("wait...\n");
        pthread_cond_wait(&b->notempty,&b->lock);
    }

    /* 读取数据，移动指针 */
    data=b->buffer[b->readpos];
    b->readpos++;
    if (b->readpos>=BUFFER_SIZE)
    {
        b->readpos=0;
    }

    b->size--;

    /* 设置缓冲区未满的条件变量 */
    pthread_cond_signal(&b->notfull);
    pthread_mutex_unlock(&b->lock);
    return data;
}

#define OVER (-1)


void *producer(void *data)
{
    int n;
    for (n=0; n < 10000; ++n)
    {
        printf ("%d --->\n",n);
        put(&buffer,n);
    }

    put(&buffer,OVER);

    return NULL;
}

void *consumer(void *data)
{
    int d;
    while(1)
    {
        d=get(&buffer);
        if (d==OVER)
        {
            break;
        }

        printf ("--->%d\n",d);
    }
    return NULL;
}

int  main(int argc, char *argv[])
{
    pthread_t th_a,th_b;
    void* retva;
    Init_Buffer(&buffer);
    
    pthread_create(&th_b,NULL,consumer,0);
    sleep(1);
    pthread_create(&th_a,NULL,producer,0);
    

    pthread_join(th_a,&retva);
    pthread_join(th_b,&retva);
    return 0;
}
