#include"ytsSocket.h"


/* 向对端发送数据，由共享缓冲区中读取数据 */
int SendMessage(int socketFd,char * sendBuffer)
{
    

    char len[5];
    int sendLen=strlen(sendBuffer);
    snprintf(len,sizeof(len),"%04d",sendLen);
    writen(socketFd,len,sizeof(len)-1);

    int sendRst=writen(socketFd,sendBuffer,sendLen);
    return sendRst;
}

int SendMessageClient(int socketFd,char * sendBuffer,int sendLen)
{
    char len[5];
    snprintf(len,sizeof(len),"%04d",sendLen);
    //printf("len:%s\n",len);
    if(send(socketFd,len,sizeof(len)-1,0)<0)
    {
        perror("send :");
        return -1;
    }

    int sendRecv;
    if((sendRecv=send(socketFd,sendBuffer,sendLen,0))<0)
    {
        perror("send sendBuffer:");
        return -1;
    }

    return sendRecv;
}



int ReceiveMessage(int sockFd,char recvMsg[])
{
    
    char msgSize[5];
    int sizeLen=sizeof(msgSize);
    readn(sockFd,msgSize,sizeLen-1);
    msgSize[sizeLen-1]=0;
    int msgLen=atoi(msgSize);
    
    int rcvLen=readn(sockFd,recvMsg,msgLen);
    if(rcvLen>0)
    {
        recvMsg[rcvLen]=0;      /* 如果成功接收数据，在数据结尾后加入‘0/’ */
    }
    return rcvLen;
        
}

int readn(int fd,void *vptr,int n)
{
    int nleft;
    int nread;
    char *ptr;
    ptr=(char*)vptr;
    nleft=n;
    while(nleft>0)
    {
        if( (nread=read(fd,ptr,nleft))<0)
        {
            if(errno==EINTR)//read again
            {
                nread=0;
            }
            else
            {
                
                return -1;
                
                //pthread_exit((void*)0);//end thread
            }
        }
        else if (nread==0)//receive FIN
        {
            close(fd);   //close socket
            //pthread_exit((void*)0);//end thread
            return 0;
        }
            

        nleft-=nread;
        ptr+=nread;
    }

    return (n-nleft);
}

int writen(int fd,const void * vptr,int n)
{
    int nleft;
    int nwritten;
    const char *ptr;

    ptr=(char*)vptr;
    nleft=n;
    while(nleft>0)
    {
        if( (nwritten=write(fd,ptr,nleft))<=0)
        {
            if(nwritten<0 && errno==EINTR)
                nwritten=0;
            else
            {
                return -1;
                //pthread_exit((void*)0);//end thread
            }
                
            
        }

        nleft-=nwritten;
        ptr+=nwritten;
    }
}

/* 接收数据，并向共享缓冲区中写入数据 */
int receive_message(int fd,struct prodcons*buffer)
{
    // printf ("recevie message\n");
    int rstn;                   /* 接收数据 */
    pthread_mutex_lock(&buffer->lock);
    //printf ("size:%d\n",buffer->size);
    /* 等待缓冲区未满 */
    while(buffer->size==BUFFER_SIZE)
    {
        pthread_cond_wait(&buffer->notfull,&buffer->lock);
    }

    if((rstn=ReceiveMessage(fd,buffer->buffer[buffer->writepos]))<=0)
    {
        pthread_mutex_unlock(&buffer->lock);
        return rstn;
    }
    else
    {
        buffer->writepos++;

        if(buffer->writepos>=BUFFER_SIZE)
        {
            buffer->writepos=0;
        }
        buffer->size++;
        pthread_cond_signal(&buffer->notempty);
        pthread_mutex_unlock(&buffer->lock);

        return rstn;
    }
    
        
}

/* 发送数据，读共享缓冲区 */
int send_message(int fd,struct prodcons *buffer)
{
    int rstn;                   /* 记录发送数据的个数 */
    pthread_mutex_lock(&buffer->lock);

    while(buffer->size==0)
    {
        pthread_cond_wait(&buffer->notempty,&buffer->lock);
    }

    /* 读取缓冲区中的数据，进行发送 */
    if((rstn=SendMessage(fd,buffer->buffer[buffer->readpos]))<=0)
    {
        pthread_mutex_unlock(&buffer->lock);
        return rstn;
    }  
    else
    {
        buffer->readpos++;
        if(buffer->readpos>=BUFFER_SIZE)
        {
            buffer->readpos=0;
        }
        buffer->size--;
        
        pthread_cond_signal(&buffer->notfull);
        pthread_mutex_unlock(&buffer->lock);
        return rstn;
    }
}

/* 初始化缓冲区相关数据 */
void init_buffer(struct prodcons*buffer)
{
    
    pthread_mutex_init(&buffer->lock,NULL);
    pthread_cond_init(&buffer->notempty,NULL);
    pthread_cond_init(&buffer->notfull,NULL);
    buffer->readpos=0;
    buffer->writepos=0;
    buffer->size=0;
        
}

//输出共享buffer中的信息
void print_buffer(struct prodcons*buffer)
{
                      /* 记录发送数据的个数 */
    pthread_mutex_lock(&buffer->lock);

    while(buffer->size==0)
    {
        pthread_cond_wait(&buffer->notempty,&buffer->lock);
    }

    /* 读取缓冲区中的数据，进行发送 */
    printf ("%s\n",buffer->buffer[buffer->readpos]);
    buffer->readpos++;
    if(buffer->readpos>=BUFFER_SIZE)
    {
       buffer->readpos=0;
    }
    
    buffer->size--;

    pthread_cond_signal(&buffer->notfull);
    pthread_mutex_unlock(&buffer->lock);
    
}
